                                                   
;----------------------------------------------------------------------
;
;   Filename:           rcvr_demo.asm
;   Date:               June 16th, 2004
;   File Version:       1.0
;   Assembled using:    
;
;   Author:             Jay Summet

;
;
;----------------------------------------------------------------------
;
;   Files required:
;                       p12f629.inc
;
;
;----------------------------------------------------------------------
;
;   Program Description 
;
;   
;
;----------------------------------------------------------------------

    list        p=12f629       ; list directive to define processor
    #include    <p12f629.inc>   ; processor specific variable definitions

    errorlevel  -302            ; suppress message 302 from list file


TRUE	equ	1
FALSE	equ	0






; ---------------------------------------------------------------------
; Configuration Bits (Section 9.1 Configuration Bits)
; ---------------------------------------------------------------------
;
; Data Memory Code Protection bit:
; _CPD_ON = Enabled
; _CPD_OFF = Disabled
;
; Program Memory Code protection:
; _CP_ON = Enabled
; _CP_OFF = : Disabled
;
; Brown-out Detection Enable bit:
; _BODEN_ON = Enabled
; _BODEN_OFF = Disabled
;
; GP3/MCLR pin function select:
; _MCLRE_ON = GP3/MCLR pin function is /MCLR
; _MCLRE_OFF = GP3/MCLR pin function is digital I/O, 
;              /MCLR internally tied to Vdd
;
; Power-up Timer Enable bit:
; _PWRTE_ON = Enabled
; _PWRTE_OFF = Disabled
;
; Watchdog Timer Enable bit:
; _WDT_ON = Enabled
; _WDT_OFF = Disabled
;
; Oscillator Selction bits:
; _EXTRC_OSC_NOCLKOUT = CLKOUT function on GP4 pin, RC on GP5 pin.
; _EXTRC_OSC_CLKOUT = I/O function on GP4 pin, RC on GP5 pin.
; _INTRC_OSC_CLKOUT =  Internal oscillator, CLKOUT function on GP4 pin,
;                      I/O function on GP5 pin.
; _INTRC_OSC_NOCLKOUT = Internal oscillator, I/O function on GP4 and GP5 pins.
; _EC_OSC = I/O function on GP4 pin, CLKIN on GP5 pin.
; _HS_OSC = High speed crystal/resonator on GP4 and GP5 pins.
; _XT_OSC = Crystal/resonator on GP4 and GP5 pins.
; _LP_OSC = Low power crystal on GP4 and GP5 pins.
;
;
; ---------------------------------------------------------------------

    __CONFIG    _CPD_OFF & _CP_OFF & _BODEN_OFF & _MCLRE_OFF & _PWRTE_OFF & _WDT_OFF & _INTRC_OSC_NOCLKOUT

;----------------------------------------------------------------------
; Variables  (Section 2.2 Data Memory Organization)
;----------------------------------------------------------------------

    ; Data Memory Organization (Section 2.2)
    ;
    ; Register locations 0x20 to 0x5F (64 bytes) are General Purpose 
    ; registers, implemented as static RAM and are mapped across both
    ; banks. 

    cblock  0x20

        w_temp                      ; used for context saving 
        status_temp                 ; used for context saving

        TEMP                        ; General Purpose Temp register
		TEMP1						;		"			temp1
        DATA0			            ; 1st byte of received data
        DATA1			            ; 2nd byte of received data
        DATA2			            ; 3rd byte of received data
        DATA3			            ; 4th byte of received data
        ORIGIN			            ; a reference used to increment TMRLOW
        SX1TMR			            ; LED timer (low order)
        SX2TMR			            ; LED timer (high order)
        TMRLOW			            ; pulse width timer (low order)
        TMRHIGH			            ; pulse width timer (high order)
        HIGHWDTH		            ; high pulse width
        LOWWDTH			            ; low pulse width
        STATECNTR		            ; program state counter
        BITCNTR			            ; data stream bit counter
        FLAGS			            ; flags
        COUNTR                      ; misc. counter

     

    ;; For RS232:
    TXD



    endc

;----------------------------------------------------------------------
; Defines
;----------------------------------------------------------------------

    
 ;; GPIO.

#define RXDATA     GPIO, 0            
#define RA1     GPIO, 1            
#define RA2     GPIO, 2            
#define LRNPB   GPIO, 3          
#define RA4     GPIO, 4            
#define TXport     GPIO, 5           

    ; Define for TRISA Register (Section 3.1)

   ;   PORTA Pins = xx543210
#define GPIOTRIS  b'00011111'

   

;----------------------------------------
; Program Defines
;----------------------------------------

    ; Status Counter

#define	BEGN	0x00
#define	BEGN1	0x01
#define	HEADR	0x02
#define	HEADR1	0x03
#define	HIGHP	0x04
#define	LOWP	0x05
#define	RECRD	0x06
#define	WAIT	0x07
#define	VALID	0x08
#define	IMPLMNT	0x09

    ; Flags

#define LRNFLG  FLAGS, 0            ; this flag is set when in learn mode
#define TOGFLG  FLAGS, 1            ; toggle flag
#define HLFLG   FLAGS, 2            ; high-low flag

 



;----------------------------------------------------------------------
; Program Memory
;----------------------------------------------------------------------

    ; Program Memory Organization (Section 2.1)

        ORG     0x000               ; RESET Vector

        nop                         ; for ICD use
        goto    RESET


        ORG     0x004               ; Interrupt Vector
       ; goto    Interrupt			;; RS232 interrupt routine...

        movwf   w_temp              ; save W register
        swapf   STATUS, W           ; swap status to be saved into W
        bcf     STATUS, RP0         ; ---- Select Bank 0 -----
        movwf   status_temp         ; save STATUS register

;----------------------------------------
; Interrupt Service Routine (ISR) (Section 9.4)
;
; Description: This program does not use interrupts
;
;----------------------------------------

        swapf   status_temp, W      ; swap status_temp into W, sets bank to original state
        movwf   STATUS              ; restore STATUS register
        swapf   w_temp, F
        swapf   w_temp, W           ; restore W register

        retfie


;----------------------------------------------------------------------
; Subroutine DATA_EEPROM_READ
;
; Description: To read an EEPROM data memory location, the address is
;   written to the EEADR register and set control bit RD (EECON1<0>) to
;   initiate a read. Data is available in the EEDATA register the next
;   clock cycle.
;
; Constants: none
;   
; Global Variables: none
;   
; Initialization: W contains EEPROM address to be read
;   
; Output: W contains EEPROM data
;
;----------------------------------------------------------------------

DATA_EEPROM_READ

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----
        movwf   EEADR               ; move EEPROM address in W to EEADR
        bsf     EECON1, RD          ; initiate EEPROM read
        movf    EEDATA, W           ; move data to W
        bcf     STATUS, RP0         ; ---- Select Bank 0 -----

        return


;----------------------------------------------------------------------
; Subroutine DATA_EEPROM_WRITE
;
; Description: To write an EEPROM data memory location, the address is
;   written to the EEADR register, data to the EEDATA register, then
;   execute a required sequence of instructions.
;   
; CAUTION: Interrupts are disable during this subroutine
;
; Constants: none
;   
; Global Variables: none
;   
; Initialization: Address = EEADR, Data = EEDATA
;   
; Output: none
;
;----------------------------------------------------------------------

DATA_EEPROM_WRITE

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----

        bsf     EECON1, WREN        ; EEPROM Write Enable: allow write cycles
        bcf     INTCON, GIE         ; disable global interrupts
                                    ; *** required sequence, do not alter ***
        movlw   0x55
        movwf   EECON2
        movlw   0xAA
        movwf   EECON2
        bsf     EECON1, WR          ; initiate EEPROM write
                                    ; *** end required sequence ***

        btfsc   EECON1, WR          ; has write completed?
        goto    $-1

;        bsf     INTCON, GIE         ; enable global interrupts
        bcf     EECON1, WREN        ; EEPROM Write Enable: inhibit write cycles

        bcf     STATUS, RP0         ; ---- Select Bank 0 -----

        return



;----------------------------------------------------------------------
; Initialize PICmicro (PIC12F675)
;----------------------------------------------------------------------

INITIAL

; Disable global interrupts during initialization

        bcf     INTCON, GIE         ; disable global interrupts


;----------------------------------------
; Calibrating the Internal Oscillator (Section 9.2.5.1)
; Oscillator Calibration Register (OSCCAL) (Section 2.2.2.7)
;
; A calibration instruction is programmed into the last location of
; program memory. This instruction is a RETLW XX, where the literal is
; the calibration value. The literal is placed in the OSCCAL register
; to set the calibration of the internal oscillator.

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----

        call    0x3FF               ; retrieve factory calibration value
        movwf   OSCCAL              ; update register with factory cal value

        bcf     STATUS, RP0         ;---- Select Bank 0 -----


;----------------------------------------
; GPIO
;
; Store GPTRIS value defined above into the TRISIO direction register

	 

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----
 		

        movlw   GPIOTRIS
        movwf   TRISIO               ; Write to TRISA register

        bcf     STATUS, RP0         ;---- Select Bank 0 -----




;----------------------------------------
; Comparator Module (Section 6.0)
;
; The PIC16F630/676 devices have one analog comparator. The inputs to
; the comparator are multiplexed with the RA0 and RA1 pins. There is
; an on-chip Comparator Voltage Reference that can also be applied to
; an input of the comparator. In addition, RA2 can be configured as
; the comparator output. The Comparator Control Register (CMCON)
; contains bits to control the comparator. The Voltage Reference
; Control Register (VRCON) controls the voltage reference module.

        ; Comparator Configuration (Figure 6-2)
;        bcf     CMCON, CINV         ; Comparator Output Inversion: not inverted
;        bcf     CMCON, COUT         ; Comparator Output bit: Vin+ < Vin-
;        bcf     CMCON, CIS          ; Comparator Input Switch: Vin- connectos to Cin-

        ; CM2:CM0 = 111 - Comparator Off (lowest power)
        bsf     CMCON, CM2          ; Comparator Mode bit 2
        bsf     CMCON, CM1          ; Comparator Mode bit 1
        bsf     CMCON, CM0          ; Comparator Mode bit 0

        ; VRCON (Register 6-2)
        bsf     STATUS, RP0         ; ---- Select Bank 1 -----

        bcf     VRCON, VREN         ; CVref circuit: powered down, no Idd drain

;        bcf     VRCON, VRR          ; CVref Range Selection: High Range

;        bcf     VRCON, VR3          ; CVref value selection bit 3
;        bcf     VRCON, VR2          ; CVref value selection bit 2
;        bcf     VRCON, VR1          ; CVref value selection bit 1
;        bcf     VRCON, VR0          ; CVref value selection bit 0

        bcf     STATUS, RP0         ;---- Select Bank 0 -----



;----------------------------------------
; TIMER1 Module with Gate Control (Section 5.0)
;


;----------------------------------------
; OPTION Register (OPTION_REG) (Section 2.2.2.2)
; TIMER0 Module (Section 4.0)
;
; The OPTION_REG contains control bits to configure:
;   Weak pull-ups on GPIO (see also WPU Register above)
;   External RA2/INT interrupt
;   TMR0
;   TMR0/WDT prescaler

        bsf     STATUS, RP0         ; ---- Select Bank 1 -----

        bsf     OPTION_REG, NOT_GPPU ; GPIO pull-ups: disabled

        bsf     OPTION_REG, INTEDG  ; Interrupt Edge: on rising edge of RA2/INT pin

        bcf     OPTION_REG, T0CS    ; TMR0 Clock Source: internal instruction cycle (CLKOUT)
        bcf     OPTION_REG, T0SE    ; TMR0 Source Edge: increment low-to-high transition on GP2/T0CKI pin

        bcf     OPTION_REG, PSA     ; Prescaler Assignment: assigned to TIMER0

        ; TMR0 Prescaler Rate: 1:4
        bcf     OPTION_REG, PS2     ; Prescaler Rate Select bit 2
        bcf     OPTION_REG, PS1     ; Prescaler Rate Select bit 1
        bsf     OPTION_REG, PS0     ; Prescaler Rate Select bit 0

        bcf     STATUS, RP0         ;---- Select Bank 0 -----





  bsf     INTCON, GIE         ; re-enable global interrupts


        retlw   0                   ; exit INITIAL 


;----------------------------------------------------------------------
; Subroutine: TIMER
;   
; Description: Continually updates two higher order timers (SX1TMR and
;   SX2TMR) for use in LED timing.
;   
; Constants: none
;   
; Global Variables: SX1TMR, SX2TMR
;   
; Initialization: none
;   
; Output: SX1TMR, SX2TMR
;   
;----------------------------------------------------------------------

TIMER
        btfss   TOGFLG              ; TOGGLE forces this routine to spend
         goto   TIMER1              ;   1/2 of TMR0 in TIMER and 1/2 in TIMER1.

        movlw   B'01111111'
        addwf   TMR0, W             ; TOGGLE toggles back and forth to a   
        btfss   STATUS, C           ;    one the rate TMR0 overflows.
         retlw  0                   ; TMR0 overflow ~= 1ms (2^8*4us)

        bcf     TOGFLG
        incfsz  SX1TMR, F           ; SX1TMR resolution ~= 1ms
         retlw  0                   ; SX1TMR overflow ~= 0.25sec (2^8*1ms) 

        incf    SX2TMR, F           ; SX2TMR resolution ~= 0.25sec
        retlw   0                   ; SX2TMR overflw ~= 1min (2^8*0.23sec)  

TIMER1
        movlw   B'01111111'         ; Timer routine spends half its time
        addwf   TMR0, W             ;   in TIMER1 waiting to set TOGGLE
        btfsc   STATUS, C           ;   to one again
         retlw  0

        bsf     TOGFLG
        retlw   0


;----------------------------------------------------------------------
; Subroutine: CLOCK
;   
; Description: Continually updates TMRLOW and TMRHIGH.
;   
; Constants: none
;   
; Global Variables: ORIGIN
;   
; Initialization: ORIGIN
;   
; Output: TMRLOW, TMRHIGH
;   
;----------------------------------------------------------------------

CLOCK
        movf    ORIGIN, W           ; TMRLOW is updated based on time passed since ORIGIN was set.
        subwf   TMR0, W             ; TMRLOW resolution ~= 4us (like TMR0)  
        addwf   TMRLOW, F           ; TMRLOW overflow ~= 1ms (2^8*4ms)
        btfsc   STATUS, C           ; TMRHIGH resolution ~= 1ms
         incf   TMRHIGH, F          ; TMRHIGH overflow ~= 0.24sec (2^8*1ms)

        nop                         ; NOP and subtraction commands ensure 
        nop                         ;   ORIGIN equals TMR0 as called upon
        nop                         ;   in line 2 of CLOCK.
        movlw   2
        subwf   TMR0, W 
        movwf   ORIGIN              ; (ORIGIN must be updated to equal the value
                                    ;   of TMR0 at time of operation with ORIGIN.)
        retlw   0




;----------------------------------------------------------------------
; Subroutine: SETWATCH
;   
; Description: Initialize the pulse width timer registers.	
;   
; Constants: 
;   
; Global Variables: 
;   
; Initialization: none
;   
; Output: ORIGIN
;   
;----------------------------------------------------------------------

SETWATCH
        movf    TMR0, W             ; record TMR0's value in ORIGIN 
        movwf   ORIGIN
        clrf    TMRLOW              ; clear the low and high order timers
        clrf    TMRHIGH

        retlw	0





;; ----------------
;; SENDByte
;;
;; Sends a byte of data, in W, at 19200 bps
;; Uses TEMP1, calls T_Wait
;;
;;------------------

SENDByte
	movwf	TXD		; store in data register
	bcf	TXport		; start bit
   	call	T_Wait

	movlw	0x08
	movwf	TEMP1		; load number of bits to send, 



_SEND_Loop
	btfsc TXD,0		; Yes, we could do this in 3 instructions, but at these
	bsf	TXport		; speeds we don't want the pin toggleing high/low at all!
	btfss	TXD,0
	bcf	TXport
	rrf	TXD,f
	call	T_Wait
	decfsz	TEMP1,f
	goto	_SEND_Loop
	bsf	TXport		; stop bit
	call	T_Wait
	call   T_Wait
	call	T_Wait			;; Only 2 works for me, if things go pear shaped, try adding another call to T_Wait...
  RETURN
	
;;--------------------
;; T_Wait
;; 
;; Delay function used to send bits over the serial line at 19200
;; 
;;--------------------
T_Wait
   movlw	0x0C		; 
	movwf	TEMP		; total wait cycle until next
	goto	X_Wait		; bit: 19200 baud ==> 52 us


X_Wait
  	decfsz	TEMP,1		; WAIT LOOP
	goto	X_Wait
	RETURN


;*********************************************************************************************************
;  
;  
;
; Configure TX Pin as output, make sure TX Pin Comes up in high state on Reset
; Pause a few ms with the TX pin in a High state to allow UARTS to recognize us...
;
;*********************************************************************************************************
						
InitSerialPort:
;
                     
 ;; NOTE: the TXport must already be set as an output!
	bsf     TXport                             ; make sure TX Pin is high on powerup, use RB Port Pullup  
	movlw 0xFF
	movwf TEMP1
	;; Make sure that the port is in a High state for a few ms before we let the rest of the
	;; program continue. This allows slow UARTS to realize that data is about to appear...
	;; In my tests, it only required 6 calls to T_Wait, but perhaps other UARTS are slower than
 	;; mine....Realtime, it's negligible, and as it only happens once at the init stage...
Init_Serial_Loop
	call T_Wait
	decfsz TEMP1,f
	goto Init_Serial_Loop

 ;; Send a characteristic (mostly for debugging purposes) to indicate that
 ;; we are alive and ready to send data...
  movlw	0x4A
  call SENDByte
  movlw 0x61
  call SENDByte
  movlw 0x79
  call SENDByte
  movlw 0x0D
  call SENDByte
  movlw 0x0A
  call SENDByte

	return

;----------------------------------------------------------------------
;----------------------------------------------------------------------
; Main Program
;----------------------------------------------------------------------
;----------------------------------------------------------------------

RESET
        clrf    FLAGS               ; clear flags
        clrf    GPIO               ; initialize PORTA inputs and outputs
  
       
        movlw   BEGN                ; setup the state counter to call BEGIN 
        movwf   STATECNTR



 
 
      call    INITIAL
      call    InitSerialPort

MAIN



;; for TESTING:
 ;  movlw 0x4D ;; capital M
 ;  call SENDByte

        call    TIMER
        call    CLOCK
      

        movlw   HIGH STATEM
        movwf   PCLATH
        movf    STATECNTR, W        ; Mask out the high order bits of 
        andlw   B'00001111'         ;   STATECNTR (a noise guard)

        addwf   PCL, F              ; The program clock (PCL) is incre-
STATEM
        goto    BEGIN               ;   mented by STATECNTR in order
        goto    BEGIN1              ;   to go to the appropiate routine
        goto    HEADER	
        goto    HEADER1
        goto    HIGHPLSE
        goto    LOWPULSE
        goto    RECORD
        goto    WAIT4END
        goto    VALIDATE
        goto    IMPLEMNT
        goto    RESET               ; These RESET commands correct
        goto    RESET               ;   erroneous values of STATECNTR   
        goto    RESET               ;   not caught by the mask above.    
        goto    RESET
        goto    RESET
        goto    RESET


;----------------------------------------------------------------------
; RECEIVE SEQUENCE
;----------------------------------------------------------------------

;*********************** CODE WORD FORMAT ****************************
;    _   _   _   _               _    __   _        __   _           _
;   | | | | | | | |             | |  |  | | |      |  | | |         | 
;   | |_| |_| |_| |_____________| |__|  |_| |_/  /_|  |_| |_________|  
;                                            /  /   
;   |--PREAMBLE---|---HEADER----|---------DATA------------|--GUARD--|
;                                                            TIME
;*********************************************************************

;******************** RECEIVED DATA BREAKDOWN ************************
;
;bytes |        DATA1          |        DATA0          |
;bits  | 7| 6| 5| 4| 3| 2| 1| 0| 7| 6| 5| 4| 3| 2| 1| 0| 
;desc. |S2|S1|S0|S3| 0| 0|      SERIAL NUMBER 3        | 
;
;bytes |        DATA3          |        DATA2          |
;bits  | 7| 6| 5| 4| 3| 2| 1| 0| 7| 6| 5| 4| 3| 2| 1| 0| 
;desc. |                    COUNTER                    |
;
;* S3 Not used
;
;*********************************************************************

;----------------------------------------
; BEGIN
;	This function looks for a possible start to the data stream.	
;	
;		Input Variables:  
;			RXDATA
;		Output Variables: 
;			none

BEGIN

;; for TESTING:
  ; movlw 0x42 ;; capital B
  ; call SENDByte

        btfsc   RXDATA
         incf   STATECNTR, F        ; Make state BEGIN1
        goto    MAIN

BEGIN1

;; for TESTING:
  ; movlw 0x42 ;; capital B
  ; call SENDByte

        btfsc   RXDATA
         goto   MAIN

        call    SETWATCH
        incf    STATECNTR, F        ; Make state HEADER

        goto    MAIN


;----------------------------------------
; HEADER
;	Detects a valid header.	
;	
;		Input Variables:  
;			RXDATA
;		Output Variables: 
;			none

HEADER	

;; for TESTING:
;   movlw 0x48 ;; capital H
;   call SENDByte


        btfsc   RXDATA              ; The program loops here until 1.25ms
         goto   RESTART             ;   passes and if the data is still

        btfss   TMRHIGH, 0          ;   low.  If both hold true -> HEADER1.
         goto   MAIN                ; 1.25ms occurs when:

        movlw   D'64'               ;   TMRHIGH = 1 ~= 2^8*4us = 1ms
        andwf   TMRLOW, W           ;   TMRLOW = 64 ~= 64*4us = 0.25ms
        btfsc   STATUS, Z
         goto   MAIN

	


        incf    STATECNTR, F        ; Make state HEADER1

        goto    MAIN

HEADER1

        movlw   D'6'                ; If the data goes high before 6ms 
        subwf   TMRHIGH, W          ;   then the header is valid, else
        btfss   STATUS, C           ;   restart.	
         goto    HEADER2            ; TMRHIGH = 6 = 6*1ms = 6ms	

        goto    RESTART

HEADER2
        btfss   RXDATA
         goto   MAIN

        call    SETWATCH		
        movlw   D'32'               ; Initiate BITCNTR to 32 in order to
        movwf   BITCNTR             ;   receive 32 bits of the data stream.
        incf    STATECNTR, F        ; Make state HIGHPLSE

        goto    MAIN


;----------------------------------------
; HIGHPLSE
;	Times the width of high pulses.	
;	
;		Input Variables:  
;			RXDATA
;		Output Variables: 
;			none


HIGHPLSE
        btfsc   TMRHIGH, 0          ; If TMRLOW overflows then RESTART
         goto   RESTART

        btfsc   RXDATA		
         goto   MAIN

        movf    TMRLOW, W           ; Move the pulse width value to 
        movwf   HIGHWDTH            ;   HIGHWDTH for later calculations.
        call    SETWATCH
        incf    STATECNTR, F        ; Make state LOWPULSE

        goto    MAIN


;----------------------------------------
;  LOWPULSE
;	Times the width of low pulses.	
;	
;		Input Variables:  
;			none
;		Output Variables: 
;			none


LOWPULSE
        btfsc   TMRHIGH, 0          ; If TMRLOW overflows then make
         goto   LOW2                ;   state HEADER.

        btfss   RXDATA
         goto   MAIN

        movf    TMRLOW, W           ; Move the pulse width value to 
        movwf   LOWWDTH             ;   LOWWDTH for later calculations.
        call    SETWATCH 
        incf    STATECNTR, F        ; Make state RECORD

        goto    MAIN

LOW2
        movlw   HEADR
        movwf   STATECNTR           ; Make state HEADER if lowpulse is
                                    ;   too long.
        goto    MAIN


;----------------------------------------
;  RECORD
;	Records each bit as it comes in from the data stream.	
;	
;		Input Variables:  
;			RXDATA
;		Output Variables: 
;			DATA0
;			DATA1
;			DATA2
;			DATA3

RECORD	
        movf    HIGHWDTH, W
        subwf   LOWWDTH, W          ; The state of the carry bit after 
        rrf     DATA3, F            ;   this operation reflects the data
        rrf     DATA2, F            ;   logic.  This is then rotated
        rrf     DATA1, F            ;   into the storage bytes.
        rrf     DATA0, F
        movlw   HIGHP
        movwf   STATECNTR
        decfsz  BITCNTR, F
         goto   MAIN

        movlw   D'4'                ; Starting here and including RECORD1
        movwf   COUNTR              ;   a check is made to make sure that

        movlw   DATA0               ;   the data is not composed entirely
        movwf   FSR                 ;   of 1s.

RECORD1
        movlw   0xFF		
        xorwf   INDF, W             ; Use indirect referencing  to point to
        btfss   STATUS, Z           ;   DATA0 -- DATA3 on subsequent loops 
         goto   RECORD2             ;   in RECORD1.

        incf    FSR, F
        decfsz  COUNTR, F
         goto   RECORD1

        goto    RESTART

RECORD2
        movlw   WAIT		
        movwf   STATECNTR           ; Make state WAIT4END

        goto    MAIN


;----------------------------------------
;  WAIT4END
;	Wait for the guard time at the end of the code word before	
;	attempting to receive another code word.
;
;		Input Variables:  
;			RXDATA
;		Output Variables: 
;			none


WAIT4END
        btfsc   HLFLG               ; HIGHLOW is set to indicate that the 
         goto   WAIT1               ;   data has transitioned from a high

        btfsc   RXDATA              ;   to a low.  
         goto   MAIN

        call    SETWATCH
        bsf     HLFLG
WAIT1
        btfss   RXDATA
         goto   WAIT2

        bcf     HLFLG

        goto    MAIN

WAIT2

        btfss   TMRHIGH, 3          ; If the low period is greater than

         goto   MAIN                ;   8ms (2^3*1ms) then the guard time 

        bcf     HLFLG               ;   has been reached.
        incf    STATECNTR, F        ; Make state VALIDATE

        goto    MAIN


;----------------------------------------
;  VALIDATE
;	Checks that the transmission received is from the valid
;	transmitter.
;
;		Input Variables:  
;			DATA0
;			DATA1
;		Output Variables: 
;			none


VALIDATE

   ;; for TESTING:
  ;; movlw 0x56 ;; capital V
  ;; call SENDByte

        movlw   0x00                ; EEPROM Address = 0x00
        call    DATA_EEPROM_READ    ; Get stored DATA0, EEDATA -> W

        xorwf   DATA0, W            ; compare with received DATA0
        btfss   STATUS, Z           ; If first byte checks out then 
         goto   RESTART             ;   continue, else restart

        incf    STATECNTR, F        ; Make state IMPLEMENT

        goto    MAIN


;----------------------------------------
;  IMPLEMNT
;	Implements the outputs specified by the received code word.
;	
;		Input Variables:  
;			DATA1
;		Output Variables: 
;			S0
;			S1
;			S2


IMPLEMNT


	
	movfw DATA0			;; Sensor Board Serial Number:
    call SENDByte
	
	;; DATA1 - Sensor board flags, currently not used...
	movfw DATA1
	call SENDByte

	movfw DATA2
	call SENDByte
    movfw DATA3
    call SENDByte

	;; CR/LF -
  movlw 0x0D
  call SENDByte
  movlw 0x0A
  call SENDByte


        goto    RESTART	


;----------------------------------------
;  RESTART
;	Sets the State Counter to BEGIN so that the receive sequence 
;	is restarted.
;
;		Input Variables:  
;			none
;		Output Variables: 
;			none


RESTART
        movlw   BEGN                ; restart receive sequence and return
        movwf   STATECNTR           ;   to MAIN

        goto    MAIN






;----------------------------------------------------------------------
; Data EEPROM Memory (Section 8.0)
;
; PIC12F629/675 devices have 128 bytes of data EEPROM with address
; range 0x00 to 0x7F.

        ; Initialize Data EEPROM Memory locations
   ;; 0x73 is our default sensor board serial number!

        ORG     0x2100
        DE      0x73, 0x43, 0x02, 0x03

;----------------------------------------------------------------------
; Calibrating the Internal Oscillator (Section 9.2.5.1)
; Oscillator Calibration Register (OSCCAL) (Section 2.2.2.7)
;
; The below statements are placed here so that the program can be
; simulated with MPLAB SIM or emulated with the ICD2 or ICE-2000.  
;
; The programmer (PICkit or PROMATE II) will save the actual OSCCAL 
; value in the device and restore it. The value below WILL NOT be
; programmed into the device.

        org     0x3ff
        retlw    0x40

;----------------------------------------------------------------------
        end                         ; end of program directive
;----------------------------------------------------------------------
