/*
 * Implements the "Tracker" class
 *
 */

#include "Tracker.h"
#include <iostream> 		// for NULL
#include <assert.h>		// for Assert.

Tracker::Tracker(float maxX, float maxY)
{
  tracker_max_x = maxX;
  tracker_max_y = maxY;

  numHistory = 0;
  
}

void Tracker::reset(void)
{
	numHistory = 0;
}

void Tracker::observation(float X, float Y, float error, int timestep)
{
	// Keep count.
	if (numHistory < HISTORY_SIZE)
		numHistory++;

	// Move previous history entires down by one.
	for(int i = HISTORY_SIZE-1; i > 0; i--)
	{
	  history[i] = history[i-1];
	}

	// Add current entry at the top of the list.
	history[0].x = X;
	history[0].y = Y;
	history[0].error = error;
	history[0].timestep = timestep;
}

float Tracker::getTrackerPosition(int timestep, float *X, float *Y, float* size)
{
 // This empty method should be overidden by subclasses!
  printf("ERROR! Tracker.getTrackerPosition should have been overridden!\n");
}

// Protected methods:

int Tracker::getNumHistory()
{ 
	return(numHistory);
}

h_ent* Tracker::getHistory(int p)
{
 if (p < 0 || p >= numHistory)
	 return(NULL);

 return(&history[p]);
}


// Returns the average time difference between each of the last
// N samples.
int Tracker::getAverageSampleTimeDifference(int n)
{

	assert(n >= 2);
	assert(n < HISTORY_SIZE);

	int sum = 0;
	for(int i = 0; i < n-1; i++)
	{
	   h_ent* t1 = getHistory(i);
	   h_ent* t2 = getHistory(i+1); 
	   int diff = t1->timestep - t2->timestep;
	   sum += diff;
	}
      
	// Calculate and return the average.
	//
	int r_value = (int) ( (float)sum / (float)(n-1) );

	return( r_value  );
	
}

// Checks the bounds (resetting to max/min) on the tracker location.
//
void Tracker::checkBounds(float* X, float* Y)
{
 // Check x:
 if (*X > tracker_max_x)
	 *X = tracker_max_x;
 if (*X < 0.0)
	 *X = 0.0;

 // Check y:
 if (*Y > tracker_max_y)
	 *Y = tracker_max_y;
 if (*Y < 0.0)
	*Y = 0.0; 
}
