
/* Tracker.h
 *  - Defines the "tracker" class.
 */

#ifndef TRACKER_142413
#define TRACKER_142413

#define HISTORY_SIZE 20

#define DEFAULT_TRACKER_SIZE 2.5
#define MIN_TRACKER_SIZE 1.5
#define MAX_TRACKER_SIZE 16.0

   struct h_ent {
           float x;
           float y;
           float error;
           int timestep;
           };


class Tracker {

	//  Current location & size of the tracker, as recommended by
	//  this class the last time it's recommendation was sought.
	float tracker_x;
        float tracker_y;
	float tracker_size;


	h_ent history[HISTORY_SIZE];
	int numHistory;
	
	// Maximum values (min is assumed to be zero)
	float tracker_max_x;
	float tracker_max_y;
 public:

  // constructer
  Tracker(float maxX, float maxY);

  // Every time an observation is made, this method is called
  // Postion of center of observation area is X,Y, radius is reported
  // with "error", and timestamp is when the observation was made.
  virtual void observation(float X, float Y, float error, int timestep);

  // When the system is ready to put up a new tracker, it will
  // ask the Tracker object where it should be placed.
  // The "timestep" is the current timestep, while the X,Y,Size
  // pointers are used as output variables to return the answer.
  //
  // The return value is a measure of how well we think we are doing.
  // This number drops to zero as we loose the sensor, and goes towards
  // 1.0 as we track it. 
  virtual float getTrackerPosition(int timestep, float *X, float *Y, float *size);


  // When the tracker total loses the sensor, we should reset it.
  virtual void reset();

 protected:
  // Returns the historical entry at location (position), or NULL if
  // it does not exist.
  h_ent* getHistory(int position);
  int getNumHistory();


  // Returns the Average timedifference between the first N history
  // items.
  int getAverageSampleTimeDifference(int n);

  // Checks the bounds (resetting to max/min) on the tracker location.
  //
  void checkBounds(float* X, float* Y);
};


#endif // TRACKER_142413
