//Open GL - Generate Gray Codes... 
//
//
//
//gcc -o Hello_World Hello_world.c -lX11 -lMesaGL -lMesaGLU -lMesatk -lm
//
#include <stdio.h> // for printf call
#include <stdlib.h> // for exit() call.
#include <strings.h>
#include <string.h>
#include <unistd.h>
#include <GL/glut.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <termios.h>
#include <sys/types.h>
#include <assert.h>
#include <math.h>
#include <limits.h>
#include "tga.c"
#include "Tracker.h"
#include "StaticTracker.h"
#include "StaticTrackerVS.h"
#include "LinearTracker.h"

#define BAUDRATE B19200
#define MODEMDEVICE "/dev/ttyUSB0"
#define _POSIX_SOURCE 1 /* POSIX compliant source */



#define MIN_CONFIDENCE_TO_TRACK  0.10

// This variable determines if we are tracking (true) or if we are
// using the localization pattern on the full screen (false);
bool  TrackingNow;

// Pointer to the tracking object we use by default.
Tracker* ourTracker;
// Keeps track of where the tracker is between calls to the
// TRACKER object.
//

// Stores the very current location of the tracker, right now!
float trackerSizeC = 1.5;
float trackerXc = 1.5;
float trackerYc = 1.5;

// Stores the tracker location/size
float trackerSize = 1.5;
float trackerX = 0.0;
float trackerY = 0.0;



const float segmentFactor = 3.0; // how much bigger are the segments?

// Holds handle to open serial port, and old terminal settings...
int fd;
struct termios oldtio;



int bit = 0;
int repeatCount = 1; 


// The "repeatInverse" flag is used to indicate if we should be repeating
// the inverse of each bit immidately after sending it.
//  The repeatInverseCount = 1 means that we should be sending the inverted bit.
//int repeatInverse = 0;
//int repeatInverseCount = 0;


int timeOfLastRedraw = 0;

//FPS variables...
int fpsFlag = 1; // no FPS by default.
int frame=0,TrackerScan=0,time,timebase=0; char fpsString[40];// FPS counter...
float fps,tps; char *c;
int font=(int)GLUT_BITMAP_8_BY_13;

int lastFramingBit = 0;

enum menuCodes {TRACKER_STATIC, TRACKER_STATIC_VS, TRACKER_LINEAR, 
		CROSSHAIRS_ON, CROSSHAIRS_OFF,PHOTO_ON,PHOTO_OFF,QUIT};


int CrosshairsState = CROSSHAIRS_OFF;
int PhotoStatus = PHOTO_OFF;


void init(void); // function header for Init.
int getBitSEC_ECC(unsigned short databits, int bitPosition);




void InitSerialPort()
{
      int res;
      struct termios newtio;
      fd = open(MODEMDEVICE, O_RDWR | O_NOCTTY );
      if (fd <0) {perror(MODEMDEVICE); exit(-1); }

      tcgetattr(fd,&oldtio); /* save current port settings */
      bzero(&newtio, sizeof(newtio));
      newtio.c_cflag = BAUDRATE |  CS8 | CLOCAL | CREAD;
      newtio.c_iflag = IGNPAR;
      newtio.c_oflag = 0;

 /* set input mode (non-canonical, no echo,...) */
										        newtio.c_lflag = 0;
       	newtio.c_cc[VTIME]    = 0;   /* inter-character timer unused */
        newtio.c_cc[VMIN]     = 0;   /* blocking read until 5 chars received */
        tcflush(fd, TCIFLUSH);
        tcsetattr(fd,TCSANOW,&newtio);

}

int GetPacket(char* outputArray,int maxBytes)
{
  int res;
  static int bufpointer = 0;
  static char buf[255];
  char inChar;

	 assert(maxBytes < 255);

	 res = 1;
         while (res ==1)
     	 {
	   res = read(fd,&inChar,1);   /* try to read a char... */
	    if (res ==1)
            {
	     buf[bufpointer] = inChar;
	     if (buf[bufpointer] == '\n')
	      {
	         buf[bufpointer+1] = 0; // so we can strncpy
		 bufpointer = 0;
	         // Copy bufpointer to outputArray and return the
	         // length of the buffer.
	         strncpy(outputArray,buf,maxBytes);
	         return(strlen(buf));

                }
	     // check to see if our input is too long...
	    if (bufpointer > maxBytes)
	    {
	       bufpointer = -1;
	    }
	    bufpointer++;
       } // end if res == 1
      } // end while res == 1

   return(0);
}






void CloseSerialPort()
{
      tcsetattr(fd,TCSANOW,&oldtio); // restore old settings...
      close(fd);
}

void exit_cleanup()
{
	 
 CloseSerialPort();
 printf("Exiting cleanly...\n");
 exit(0);

}



void processMenuEvents(int option) {

  switch (option) {
	  case QUIT:
		  exit_cleanup();
		  break;
	  case TRACKER_LINEAR:
		  ourTracker->reset();
		  delete(ourTracker);
		  ourTracker = new LinearTracker(32.0,32.0);
		  break;
	  case TRACKER_STATIC_VS:
		  ourTracker->reset();
		  delete(ourTracker);
		  ourTracker = new StaticTrackerVS(32.0,32.0);
		  break;
          case TRACKER_STATIC:
		  ourTracker->reset();
		  delete(ourTracker);
		  ourTracker = new StaticTracker(32.0,32.0);
	          break;
	  case CROSSHAIRS_ON:
		  CrosshairsState = CROSSHAIRS_ON;
		  break;
	  case CROSSHAIRS_OFF:
		  CrosshairsState = CROSSHAIRS_OFF;
		  break;
	  case PHOTO_ON:
		  PhotoStatus = PHOTO_ON;
		  break;
	  case PHOTO_OFF:
		  PhotoStatus = PHOTO_OFF;
	  default:
		  break;
  } // end switch.

}

void createGLUTMenus(){
	int menu;
        int submenuTrackerType;
	int submenuDemoGlitter;

	submenuTrackerType = glutCreateMenu(processMenuEvents);
	glutAddMenuEntry("Static Tracker",TRACKER_STATIC);
        glutAddMenuEntry("Static Tracker -Variable Size",TRACKER_STATIC_VS);
	glutAddMenuEntry("Linear Tracker",TRACKER_LINEAR);


	submenuDemoGlitter = glutCreateMenu(processMenuEvents);
  	glutAddMenuEntry("Crosshairs ON",CROSSHAIRS_ON);
 	glutAddMenuEntry("Crosshairs OFF",CROSSHAIRS_OFF);
	glutAddMenuEntry("Photo ON",	PHOTO_ON);
	glutAddMenuEntry("Photo OFF", PHOTO_OFF);
	

	//create the meu and
	//tell glut that "processMenuEvents" will
	//handle the events
	menu = glutCreateMenu(processMenuEvents);
	glutAddSubMenu("Tracker",submenuTrackerType);
	glutAddSubMenu("Demo Glitter",submenuDemoGlitter);
	glutAddMenuEntry("Quit",QUIT);
	glutAttachMenu(GLUT_RIGHT_BUTTON);
	
}


static GLfloat spin  = 0.0;

int getBitSEC_ECC(unsigned short databits, int bitPosition)
{
	        // Load the databits into an array, one bit per element.
		  unsigned char d[16];
		  unsigned char c[21];
		  unsigned short int temp;

		   unsigned short int mask = 0x0001;
		    int i,j;
		     for(i = 0; i < 16; i++)
		      {
	               temp = 0x00;	
	               temp = databits & mask; // Grab one bit.
	               mask <<= 1; // advance to the next bit in the mask.
		    // Shift the data point back to the first bit position.
		             for(j = i; j>0; j--)
		                {
		                 temp >>= 1;
		                 }
			     d[i] = temp ;
		     }


	 // Move the data bits into the appropriate codeword positions
     c[2] = d[0];
     c[4] = d[1]; c[5]=d[2]; c[6] = d[3];
     c[8] = d[4]; c[9]=d[5]; c[10]=d[6]; c[11]=d[7]; c[12]=d[8]; c[13] = d[9];
     c[14] = d[10];
     c[16] = d[11]; c[17]=d[12]; c[18]=d[13]; c[19]=d[14]; c[20]=d[15];

     // Calculate the check bits.
     
     c[0] = c[2] ^ c[4] ^ c[6] ^ c[8] ^ c[10] ^ c[12] ^ c[14] ^ c[16] ^ c[18] ^ c[20];
     c[1] = c[2] ^ c[5] ^ c[6] ^ c[9] ^ c[10] ^ c[13] ^ c[14] ^ c[17] ^ c[18];
     c[3] = c[4] ^ c[5] ^ c[6] ^ c[11]^ c[12] ^ c[13] ^ c[14] ^ c[19] ^ c[20];
     c[7] = c[8] ^ c[9] ^ c[10] ^c[11] ^ c[12] ^ c[13] ^ c[14];
     c[15] = c[16] ^ c[17] ^ c[18] ^ c[19] ^ c[20];

     // return the appropriate bit...
     for(i = 0; i < 21; i++)
     {
	     if (i == bitPosition)
		     return(c[i]);
     }
     return(0); //should never be hit...
}

 

int getBit(int x, int y, int frame)
{

	 int internalBit = 0;
	 int positionNum = 0;
	 int n,v,dig,set,k;
	 unsigned short int temp;

		 // This one uses 21 bits to transfer 16 bits of data.
		 positionNum = x * 32 + y;
		 // We can use the upper six bits for whatever we want...
		 temp = (unsigned short int)positionNum;
		 temp |=0xA800;

		 //if (x == 0 && y == 0)
		//	 printf("Position is: %d\n",temp);

		return (getBitSEC_ECC(temp,frame));


 // If we ever get a filled screen, it's because this
 // return was executed!
  return(0);
}

int getTFcolor(int segment,int bit, int frame)
{

	if (bit > 7)
		return(0);

//	assert(bit >= 0 && bit <= 7);
	assert(frame == 1 || frame == 0);

	// The first two bits are framing bits, 00...11...00...11..00 etc...
	// shared by all segments. Only requirement is that they alternate
	// at the begining of each frame (so the start bits for frame 2 are
	// the end bits for frame1, etc...etc...)
       if (bit < 2 )
	       return(frame);


	int bitArray[8][6] =
	{ { 0,0,0,0,0,0},		//Segment 0 - outside...not used.
   	  { 1,0,1,0,1,0},		// segment 1 - inside.  111
          { 0,1,0,1,1,0}, 	        // segment 2, 		001
	  { 0,1,1,0,0,1},		// segment 3,		010
	  { 0,1,1,0,1,0},		// segment 4,		011
	  { 1,0,0,1,0,1},		// segment 5,		100
	  { 1,0,0,1,1,0},		// segment 6,		101
	  { 1,0,1,0,0,1},		// segment 7		110
	
	};

	return( bitArray[segment][bit-2]);

}

void setSegmentColor(int segment,int bit, int frame)
{
	if ( getTFcolor(segment,bit,frame) == 0)
		glColor3f(0.0,0.0,0.0);
	else
		glColor3f(1.0,1.0,1.0);
}

void drawImage(float x, float y)
{
  
    glEnable (GL_TEXTURE_2D); /* enable texture mapping */
    glBindTexture (GL_TEXTURE_2D, 13); /* bind to our texture, has id of 13 */

    glColor3f(1.0, 1.0, 1.0);
    glBegin (GL_QUADS);
        glTexCoord2f (0.0f,0.0f); /* lower left corner of image */
        glVertex3f (x, y, 0.0f);
        glTexCoord2f (1.0f, 0.0f); /* lower right corner of image */
        glVertex3f (x+20.0f, y, 0.0f);
        glTexCoord2f (1.0f, 1.0f); /* upper right corner of image */
        glVertex3f (x+ 20.0f, y+20.0f, 0.0f);
        glTexCoord2f (0.0f, 1.0f); /* upper left corner of image */
        glVertex3f (x, y+20.0f, 0.0f);
    glEnd ();

    glDisable (GL_TEXTURE_2D); /* disable texture mapping */

}

// Draws a red & green crosshairs indicating the current location of
// the tracked object.
void drawCrossHairs(float x, float y)
{

	// vertical one is green.
       glColor3f(0.0, 1.0, 0.0);
       glRectf(x-0.1,-32.0, x+0.1,32.0);
 
       glColor3f(1.0,0.0,0.0);
       glRectf(-32.0, y-0.1,32.0,y+0.1);

}

void drawTrackingFigure(float x, float y,float size,int bit, int frame)
{


	/* This code draws a hexigonal tracking pattern with 
	 * verticies as follows: (_#_ indicate segment numbers, zero
	 * is outside of the pattern, 1 is in the center...)
	 *
		
                     8         7
                      \  _2_  /
                       \     /
                 _7_    \   /       _3_
                         2 1
             9----------3 1 0------------6
                         4 5
                _6_     /   \       _4_
		       /     \
                      /  _5_  \
                    10         11


	  - Note: Not to Scale!    */


	const float xd = (float)  0.5 * size;
        const float yd = (float)  0.5* sqrt(3) * size;

	const float sxd = (float) 0.5 * size * segmentFactor;
	const float syd = (float)  0.5 * sqrt(3) * size * segmentFactor;

       // Draw the center hexagon....segment 1
       //
       setSegmentColor(1,bit,frame);
       glBegin(GL_POLYGON);
       	 glVertex2f(x+size,y);		// 0
	 glVertex2f(x+xd,y-yd);		// 1
	 glVertex2f(x-xd,y-yd);		// 2
	 glVertex2f(x-size,y);		// 3
	 glVertex2f(x-xd,y+yd);		// 4
	 glVertex2f(x+xd,y+yd);		// 5
       glEnd();

       // Draw segment 2
       //
       setSegmentColor(2,bit,frame);
       glBegin(GL_POLYGON);
         glVertex2f(x+xd,y-yd);		// 1
	 glVertex2f(x+sxd,y-syd);	// 7
	 glVertex2f(x-sxd,y-syd);	// 8
	 glVertex2f(x-xd,y-yd);		// 2
       glEnd();
       

	//Draw segment 3...
	setSegmentColor(3,bit,frame);
	glBegin(GL_POLYGON);
 	  glVertex2f(x+size,y);				// 0
   	  glVertex2f(x+size*segmentFactor,y);		// 6
	  glVertex2f(x+sxd,y-syd);			// 7
	  glVertex2f(x+xd,y-yd);			// 1
	glEnd();

	// Draw segment 4...
	setSegmentColor(4,bit,frame);
	glBegin(GL_POLYGON);
	  glVertex2f(x+xd,y+yd);			//5
	  glVertex2f(x+sxd,y+syd);			//11
	  glVertex2f(x+size*segmentFactor,y);		//6
	  glVertex2f(x+size,y);				//0
	glEnd();

	// Draw segment 5
	setSegmentColor(5,bit,frame);
	glBegin(GL_POLYGON);
	  glVertex2f(x-xd,y+yd);			//4
	  glVertex2f(x-sxd,y+syd);			//10
	  glVertex2f(x+sxd,y+syd);			//11
	  glVertex2f(x+xd,y+yd);			//5
	glEnd();

	// Draw segment 6
	setSegmentColor(6,bit,frame);
	glBegin(GL_POLYGON);
	  glVertex2f(x-size,y);				//3
	  glVertex2f(x-size*segmentFactor,y);		//9
	  glVertex2f(x-sxd,y+syd);			//10
	  glVertex2f(x-xd,y+yd);
	glEnd();

	// Draw segment 7
	setSegmentColor(7,bit,frame);
	glBegin(GL_POLYGON);
	  glVertex2f(x-xd,y-yd);			//2
	  glVertex2f(x-sxd,y-syd);			//8
	  glVertex2f(x-size*segmentFactor,y);		//9
	  glVertex2f(x-size,y);				//3
	glEnd();
             

}

void display(void)
{
 int i,j;

 //printf("Bit: %d",bit);
 //printf("  Sample Pixel (20,20) : %d\n", getBit(20,20,bit));

  // clear all pixels.
  glClear(GL_COLOR_BUFFER_BIT);


  // Locate or track? 
  if (TrackingNow == false) 
  {
  // Draw 32x32 rectangles...
  for( i=0; i < 32; i++)
  {
     for( j = 0; j < 32; j++)
     {
	     int temp = getBit(i,j,bit);

       if (temp == 1)
       {
	               glColor3f(1.0, 1.0, 1.0);
       }
       else
	       glColor3f(0.0, 0.0, 0.0);

       glRectf( (i-16)*2 , 2*(j-16),((i-16)*2)+2,((j-16)*2)+2);
     } // end for j

   } // end for i

       // some computers don't sync redraws with vsync... (specifically mine)
       // make sure we go no faster than 60hz.
           while(glutGet(GLUT_ELAPSED_TIME) < timeOfLastRedraw + 16);

  } else  // Track the sensor! 
  {
    if (CrosshairsState == CROSSHAIRS_ON)
        drawCrossHairs((trackerXc-16)*2 +1, (trackerYc-16)*2 +1);

    if(PhotoStatus == PHOTO_ON)
        drawImage( (trackerXc-16)*2 + 4, (trackerYc-16)*2 + 4);

    drawTrackingFigure((trackerXc-16)*2 +1,(trackerYc-16)*2 +1,
		   trackerSizeC,bit,lastFramingBit);
       // some computers don't sync redraws with vsync... (specifically mine)
       // make sure we go no faster than 60hz.
           while(glutGet(GLUT_ELAPSED_TIME) < timeOfLastRedraw + 17);
			// This is a 17 because tracking pattern is
			// faster than SEC.

  }


  //glFlush();
  //glFinish();
  //
       
          timeOfLastRedraw = glutGet(GLUT_ELAPSED_TIME);
      
  glutSwapBuffers();
}

// Checks to see where the sensor says it is....
// PIC sends the following two byte payload:
//   Save the data into our bytes to send. (NOTE! We only save the 10 data bits.
//   We also add a 4 bit (0-15) number representing which sensor we got the 
//   value off of. (Currently, we only have one sensor, number zero...)^M
//   We also add 2 bits indicating what type of tracking pattern it was (01 - 
//     10 bit track. 10-3 bit track)^M
//        ;; 10 3210 9876543210^M
//        ;; MM-SSSS-DDDDDDDDDD^M
//        ;; | High    | Low   |^M
//
void locate_sensor(void)
{
	// Every frame that we haven't heard from the sensor
	// we increment this watching out for overflow!


       char buffer[10];
       int res= GetPacket(buffer,10);

       if (res > 0) // did we receive something?
       {
	if (strncmp(buffer,"Jay",3) == 0)
	{
		printf("Radio Receiver Initalized...\n");
	}
	if (buffer[0] == 's') // Serial number for this pic is 0x73=s
	{
		// Buffer[0] - pic Serial number 
		// buffer[1] - function bits (not used currently)
		// buffer[2] - High data word
		// buffer[3] - low data word

	   unsigned char patternType = buffer[2];
	   patternType  >>= 6;  // Shift six bits away, leaving top two.
	   patternType &= 0x03; // Wipe out all but bottom two bits.
	   printf("Pattern type: %d  - ",(unsigned int) patternType);

	   unsigned char sensorNumber = buffer[2];
	   sensorNumber >>=2; // shift two bits away, leave top six.
	   sensorNumber &= 0x0F; // we only want the bottom 4
	   printf("Sensor #: %d ",(unsigned int) sensorNumber);


	   // for testing
	   //unsigned int b1 = (unsigned int) buffer[2];
	   //unsigned int b2 = (unsigned int) buffer[3];
	   //printf("received bytes: %d, %d\n",b1,b2);

	   // Decode the 10 bit number:
	   unsigned short int position = (unsigned short int) buffer[2]; 
	   // Wipe out all but the last two..
	   position &= 0x03;
	   // combine the 16 bits... 
	   // // First, move the high bits up...
	   position <<= 8;

	   unsigned short int temp;
	   temp = (unsigned short int) buffer[3];
	   temp &= 0x00FF;
           position &= 0xFF00;
	   position |= temp;

	   printf("Position decoded is: %d  - ",position);	   


	  if (patternType == 1)
	  {

		if(TrackingNow == true)
		{
			printf("Threw out invalid 10 bit scan!");
			return;
		}
	   float X = (float) (position / 32);
	   float Y = (float) (position % 32);
	   printf("(X:%f,Y:%f)\n",X, Y);	
	   // Save the location in ourTracker.
	   ourTracker->observation(X,Y,1.0, glutGet(GLUT_ELAPSED_TIME));
           TrackingNow = true; // Now we go to tracking mode!
	   trackerXc = X;
	   trackerYc = Y;
	   trackerX = X;
	   trackerY = Y; 
	   
	  }

	  if (patternType == 2)
	  {

		if (TrackingNow == false)
		{
			printf("Rejected invalid 3 bit scan!\n");
			return;
		}
	
	   TrackerScan++;  // part of the FPS/tps diag code...

		printf("\nTracking-segment: %d\n",position);
		switch (position)
		{
		 case 7: // Inside the pattern.
			 ourTracker->observation(trackerX, trackerY,
				 trackerSize,glutGet(GLUT_ELAPSED_TIME));
			break;
		 case 1: // Segment 1, move tracking figure...
			ourTracker->observation(trackerX,
					trackerY - (trackerSize / 2.0),
					trackerSize,
					glutGet(GLUT_ELAPSED_TIME));
			break;
		 case 2: // Segment 2,
			ourTracker->observation(trackerX + (trackerSize /1.5),
					trackerY - (trackerSize / 1.5),
					trackerSize,
					glutGet(GLUT_ELAPSED_TIME));
			break;
		 case 3: // Segment 3
			ourTracker->observation(trackerX + (trackerSize / 1.5),
					trackerY + (trackerSize / 1.5),
					trackerSize,
					glutGet(GLUT_ELAPSED_TIME));
			break;
		 case 4: // Segment 4, move tracking figure...
			ourTracker->observation(trackerX,
					trackerY + (trackerSize / 2.0),
					trackerSize,
					glutGet(GLUT_ELAPSED_TIME));
			break;
		 case 5: // Segment 5
			ourTracker->observation(trackerX - (trackerSize / 1.5),
					trackerY + (trackerSize / 1.5),
					trackerSize,
					glutGet(GLUT_ELAPSED_TIME));
			break;
		 case 6: // Segment 6
			ourTracker->observation(trackerX - (trackerSize / 1.5),
					trackerY - (trackerSize / 1.5),
					trackerSize,
					glutGet(GLUT_ELAPSED_TIME));

			break;

		default:
			break;

		} // end case.
	  } // end receive tracking pattern.
	   
	

	} // end if serial number matched.
        else
        {
	  printf("Received garbage:%c%c%c%c\n",buffer[0],buffer[1],
						buffer[2],buffer[3]);
        }

       } // end if received something...




}

void generate_code(void)
{

	locate_sensor();
	
	//FPS counter code:
	//
	frame++;
	time=glutGet(GLUT_ELAPSED_TIME);
			
	if (time - timebase > 1000) {
		fps = frame*1000.0/(time-timebase);
		tps = TrackerScan*1000.0 / (time-timebase);
		printf("FPS:%4.2f   TPS: %4.2f\n", fps,tps);
		timebase = time;		
		TrackerScan = 0;
		frame = 0;
	}

	// Increment the bit.
	bit++;



 if (repeatCount == 0)
 {
     glutIdleFunc(0);
     bit=0;
 }


     if (TrackingNow == false)
     {
         //determine when to reset for SEC_ECC code...
	   if (bit > 20)
	   {
		   bit = 0;
		   repeatCount--;

	   }
     } else
     {		// Tracking pattern, reset every 8 bits...also recycle frame
	     if (bit > 7)
	     {
		     bit = 0;
		     repeatCount--;
		     if (lastFramingBit != 0)
			     lastFramingBit = 0;
		     else
			     lastFramingBit = 1;

		     // If we are done projecting this tracking pattern, we
		     // update the non-current tracking data now...
		     trackerSize = trackerSizeC;
		     trackerX = trackerXc;
		     trackerY =trackerYc;


		     // Get the brand new "current position"	
		     float confidence = ourTracker->getTrackerPosition(
				     		glutGet(GLUT_ELAPSED_TIME),
				     	   	&trackerXc, &trackerYc, 
						&trackerSizeC);

		     if (confidence < MIN_CONFIDENCE_TO_TRACK)
		     {
			     TrackingNow = false;
			     ourTracker->reset();
		     }

	     }
     } // end if tracking vs SEC_ECC


glutPostRedisplay();



}

void reshape(int w, int h)
{
	glViewport(0,0,(GLsizei) w, (GLsizei) h);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(-32.0,32.0,-32.0,32.0, -1.0, 1.0);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

//Keyboard handler
void keyboard(int key, int x, int y) {
	switch(key){
	case GLUT_KEY_F1:
		glutGameModeString("640x480");
		// This test always fails.. but the
		// code below seems to work for me :>
		//if (glutGameModeGet(GLUT_GAME_MODE_POSSIBLE)) {
			glutEnterGameMode();
			init();
	//	}else
	//		printf("GLUT- Full screen mode not possible");

		break;
	case GLUT_KEY_F2:
		glutFullScreen();
		break;
	case GLUT_KEY_F3:
		glutReshapeWindow(250,250);
		glutPositionWindow(100,100);
		break;
	case GLUT_KEY_F6:
		glutIdleFunc(0);
		repeatCount = 0;
		bit = 0;
		glutLeaveGameMode();
		break;
	} // end switch.

}

void mouse(int button, int state, int x, int y)
{
  switch(button) {
	  // Left button, run once...
	  case GLUT_LEFT_BUTTON:
		  if (state == GLUT_DOWN) {
			  bit = 0; // start at 0 bit.
			  repeatCount = 2; // repeat twice.
			  glutIdleFunc(generate_code);
		  }
		  break;
	  case GLUT_MIDDLE_BUTTON:
		  if (state == GLUT_DOWN) {
			  bit = 0; repeatCount = 32000;
			  glutIdleFunc(generate_code);
		  }
		  break;
	  default:
		  break;
  }
}

void init(void)
{
  if (!loadTGA("picture.tga",13))
	  printf("picture.tga not found!\n");

  glClearColor(0.0,0.0,0.0,0.0);
  glutDisplayFunc(display);
  glutReshapeFunc(reshape);
  glutMouseFunc(mouse);
  glutSpecialFunc(keyboard);

}

int main(int argc, char** argv)
{

  TrackingNow = false; // we start off in Locating mode. 
  InitSerialPort();

  // Our default tracker: 
  ourTracker = new StaticTrackerVS(32.0,32.0);

  glutInit(&argc,argv);
  glutInitDisplayMode(GLUT_RGBA|GLUT_DOUBLE);
  glutInitWindowSize(250,250);
  glutInitWindowPosition(100,100);
  glutCreateWindow(argv[0]);
  init();
  createGLUTMenus();
  glutMainLoop();
  exit_cleanup();
  return 0;
}

